/* jshint node: true */
var path = require("path");
var gulp = require("gulp");
var browserify = require("browserify");
var source = require("vinyl-source-stream");
var buffer = require("vinyl-buffer");
var uglify = require("gulp-uglify");
var async = require("async");

function b(src, dest, opts) {
  return function(callback) {
    var main  = path.join(src, opts.main || "index.js");
    var out    = opts.out || "bundle.js";
    var config = opts.browserify || {};

    var br = browserify(main, config).bundle();
    br
      .pipe(source(out))
      .pipe(buffer())
      .pipe(uglify())
      .pipe(gulp.dest(dest))
      .on("end", callback);
  };
}

/*
 * returns a gulp function to run 1 or more browserify compiles
 *
 * @param {String} src        (source of javascripts, eg. /assets/js)
 * @param {String} dest       (ouput for javascripts, eg. /public/js)
 * @param {Object|Array} opts (keys: main, out, browserify)
 * @return {Function}
 * @api public
 */

module.exports = function(src, dest, opts) {
  return function(done) {
    if (!(opts instanceof Array)) {
      opts = [opts];
    }

    async.parallel(opts.map(function(v) {
      return b(src, dest, v);
    }), done);
  };
};
